/*
 * Mana QML plugin
 * Copyright (C) 2010  Thorbjørn Lindeijer 
 * Copyright (C) 2012  Erik Schilling 
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#include "being.h"

#include "spritelistmodel.h"

#include "resource/spritedef.h"

#include <cmath>

using namespace Mana;

Being::Being(int type)
    : mType(type)
    , mId(0)
    , mWalkSpeed(0.0)
    , mAction(SpriteAction::STAND)
    , mDirection(DOWN)
    , mGender(Mana::GENDER_UNSPECIFIED)
{
    mSpriteList = new SpriteListModel(this);
}

void Being::setPosition(QPointF position)
{
    if (mPosition == position)
        return;

    mPosition = position;
    emit positionChanged();
}

void Being::setServerPosition(QPointF position)
{
    mServerPosition = position;
}

Action::SpriteDirection Being::spriteDirection() const
{
    return beingToSpriteDirection(mDirection);
}

void Being::setDirection(BeingDirection direction)
{
    if (mDirection == direction)
        return;

    mDirection = direction;
    emit directionChanged(direction);
}

void Being::setName(const QString &name)
{
    if (mName == name)
        return;

    mName = name;
    emit nameChanged();
}

void Being::setAction(const QString &action)
{
    if (mAction != action) {
        mAction = action;
        emit actionChanged();
    }
}

void Being::lookAt(const QPointF &point)
{
    if (mPosition == point)
        return;

    QPointF diff = point - mPosition;
    qreal dx = std::abs(diff.x());
    qreal dy = std::abs(diff.y());

    if (dx < dy || (dx - dy) / dx < 0.1)
        setDirection(diff.y() < 0 ? UP : DOWN);
    else
        setDirection(diff.x() < 0 ? LEFT : RIGHT);
}
